package javax.swing.plaf.synth;

import java.awt.Toolkit;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Component;
import javax.swing.plaf.UIResource;
import javax.swing.ImageIcon;
import org.xml.sax.SAXParseException;
import org.xml.sax.Locator;
import org.xml.sax.InputSource;
import javax.swing.UIDefaults;
import java.util.Locale;
import java.util.regex.PatternSyntaxException;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.plaf.DimensionUIResource;
import javax.swing.plaf.ColorUIResource;
import java.awt.Color;
import sun.reflect.misc.ReflectUtil;
import javax.swing.plaf.FontUIResource;
import java.awt.Font;
import sun.swing.plaf.synth.DefaultSynthStyle;
import org.xml.sax.Attributes;
import java.awt.Insets;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.net.URLClassLoader;
import java.net.MalformedURLException;
import java.io.IOException;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import java.text.ParseException;
import java.io.BufferedInputStream;
import javax.xml.parsers.SAXParserFactory;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.net.URL;
import java.util.Map;
import java.util.List;
import com.sun.beans.decoder.DocumentHandler;
import org.xml.sax.helpers.DefaultHandler;

class SynthParser extends DefaultHandler
{
    private static final String ELEMENT_SYNTH = "synth";
    private static final String ELEMENT_STYLE = "style";
    private static final String ELEMENT_STATE = "state";
    private static final String ELEMENT_FONT = "font";
    private static final String ELEMENT_COLOR = "color";
    private static final String ELEMENT_IMAGE_PAINTER = "imagePainter";
    private static final String ELEMENT_PAINTER = "painter";
    private static final String ELEMENT_PROPERTY = "property";
    private static final String ELEMENT_SYNTH_GRAPHICS = "graphicsUtils";
    private static final String ELEMENT_IMAGE_ICON = "imageIcon";
    private static final String ELEMENT_BIND = "bind";
    private static final String ELEMENT_BIND_KEY = "bindKey";
    private static final String ELEMENT_INSETS = "insets";
    private static final String ELEMENT_OPAQUE = "opaque";
    private static final String ELEMENT_DEFAULTS_PROPERTY = "defaultsProperty";
    private static final String ELEMENT_INPUT_MAP = "inputMap";
    private static final String ATTRIBUTE_ACTION = "action";
    private static final String ATTRIBUTE_ID = "id";
    private static final String ATTRIBUTE_IDREF = "idref";
    private static final String ATTRIBUTE_CLONE = "clone";
    private static final String ATTRIBUTE_VALUE = "value";
    private static final String ATTRIBUTE_NAME = "name";
    private static final String ATTRIBUTE_STYLE = "style";
    private static final String ATTRIBUTE_SIZE = "size";
    private static final String ATTRIBUTE_TYPE = "type";
    private static final String ATTRIBUTE_TOP = "top";
    private static final String ATTRIBUTE_LEFT = "left";
    private static final String ATTRIBUTE_BOTTOM = "bottom";
    private static final String ATTRIBUTE_RIGHT = "right";
    private static final String ATTRIBUTE_KEY = "key";
    private static final String ATTRIBUTE_SOURCE_INSETS = "sourceInsets";
    private static final String ATTRIBUTE_DEST_INSETS = "destinationInsets";
    private static final String ATTRIBUTE_PATH = "path";
    private static final String ATTRIBUTE_STRETCH = "stretch";
    private static final String ATTRIBUTE_PAINT_CENTER = "paintCenter";
    private static final String ATTRIBUTE_METHOD = "method";
    private static final String ATTRIBUTE_DIRECTION = "direction";
    private static final String ATTRIBUTE_CENTER = "center";
    private DocumentHandler _handler;
    private int _depth;
    private DefaultSynthStyleFactory _factory;
    private List<ParsedSynthStyle.StateInfo> _stateInfos;
    private ParsedSynthStyle _style;
    private ParsedSynthStyle.StateInfo _stateInfo;
    private List<String> _inputMapBindings;
    private String _inputMapID;
    private Map<String, Object> _mapping;
    private URL _urlResourceBase;
    private Class<?> _classResourceBase;
    private List<ColorType> _colorTypes;
    private Map<String, Object> _defaultsMap;
    private List<ParsedSynthStyle.PainterInfo> _stylePainters;
    private List<ParsedSynthStyle.PainterInfo> _statePainters;
    
    SynthParser() {
        this._mapping = new HashMap();
        this._stateInfos = new ArrayList();
        this._colorTypes = new ArrayList();
        this._inputMapBindings = new ArrayList();
        this._stylePainters = new ArrayList();
        this._statePainters = new ArrayList();
    }
    
    public void parse(InputStream in, DefaultSynthStyleFactory factory, URL urlResourceBase, Class<?> classResourceBase, Map<String, Object> defaultsMap) throws ParseException, IllegalArgumentException {
        if (in == null || factory == null || (urlResourceBase == null && classResourceBase == null)) {
            throw new IllegalArgumentException("You must supply an InputStream, StyleFactory and Class or URL");
        }
        assert classResourceBase == null;
        this._factory = factory;
        this._classResourceBase = classResourceBase;
        this._urlResourceBase = urlResourceBase;
        this._defaultsMap = defaultsMap;
        try {
            try {
                SAXParserFactory.newInstance().newSAXParser().parse(new BufferedInputStream(in), this);
            }
            catch (ParserConfigurationException obj) {
                throw new ParseException("Error parsing: " + obj, 0);
            }
            catch (SAXException obj2) {
                throw new ParseException("Error parsing: " + obj2 + " " + obj2.getException(), 0);
            }
            catch (IOException obj3) {
                throw new ParseException("Error parsing: " + obj3, 0);
            }
        }
        finally {
            this.reset();
        }
        return;
    }
    
    private URL getResource(String s) {
        if (this._classResourceBase != null) {
            return this._classResourceBase.getResource(s);
        }
        try {
            return new URL(this._urlResourceBase, s);
        }
        catch (MalformedURLException ex) {
            return null;
        }
    }
    
    private void reset() {
        this._handler = null;
        this._depth = 0;
        this._mapping.clear();
        this._stateInfos.clear();
        this._colorTypes.clear();
        this._statePainters.clear();
        this._stylePainters.clear();
    }
    
    private boolean isForwarding() {
        return this._depth > 0;
    }
    
    private DocumentHandler getHandler() {
        if (this._handler == null) {
            this._handler = new DocumentHandler();
            if (this._urlResourceBase != null) {
                this._handler.setClassLoader(new URLClassLoader(new URL[] { this.getResource(".") }, Thread.currentThread().getContextClassLoader()));
            }
            else {
                this._handler.setClassLoader(this._classResourceBase.getClassLoader());
            }
            for (String id : this._mapping.keySet()) {
                this._handler.setVariable(id, this._mapping.get(id));
            }
        }
        return this._handler;
    }
    
    private Object checkCast(Object o, Class obj) throws SAXException {
        if (!obj.isInstance(o)) {
            throw new SAXException("Expected type " + obj + " got " + o.getClass());
        }
        return o;
    }
    
    private Object lookup(String id, Class clazz) throws SAXException {
        if (this._handler != null && this._handler.hasVariable(id)) {
            return this.checkCast(this._handler.getVariable(id), clazz);
        }
        Object value = this._mapping.get(id);
        if (value == null) {
            throw new SAXException("ID " + id + " has not been defined");
        }
        return this.checkCast(value, clazz);
    }
    
    private void register(String str, Object value) throws SAXException {
        if (str != null) {
            if (this._mapping.get(str) != null || (this._handler != null && this._handler.hasVariable(str))) {
                throw new SAXException("ID " + str + " is already defined");
            }
            if (this._handler != null) {
                this._handler.setVariable(str, value);
            }
            else {
                this._mapping.put(str, value);
            }
        }
    }
    
    private int nextInt(StringTokenizer stringTokenizer, String s) throws SAXException {
        if (!stringTokenizer.hasMoreTokens()) {
            throw new SAXException(s);
        }
        try {
            return Integer.parseInt(stringTokenizer.nextToken());
        }
        catch (NumberFormatException ex) {
            throw new SAXException(s);
        }
    }
    
    private Insets parseInsets(String str, String s) throws SAXException {
        StringTokenizer stringTokenizer = new StringTokenizer(str);
        return new Insets(this.nextInt(stringTokenizer, s), this.nextInt(stringTokenizer, s), this.nextInt(stringTokenizer, s), this.nextInt(stringTokenizer, s));
    }
    
    private void startStyle(Attributes attributes) throws SAXException {
        String value = null;
        this._style = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("clone")) {
                this._style = (ParsedSynthStyle)((ParsedSynthStyle)this.lookup(attributes.getValue(i), ParsedSynthStyle.class)).clone();
            }
            else if (qName.equals("id")) {
                value = attributes.getValue(i);
            }
        }
        if (this._style == null) {
            this._style = new ParsedSynthStyle();
        }
        this.register(value, this._style);
    }
    
    private void endStyle() {
        int size = this._stylePainters.size();
        if (size > 0) {
            this._style.setPainters((ParsedSynthStyle.PainterInfo[])this._stylePainters.toArray(new ParsedSynthStyle.PainterInfo[size]));
            this._stylePainters.clear();
        }
        int size2 = this._stateInfos.size();
        if (size2 > 0) {
            this._style.setStateInfo((DefaultSynthStyle.StateInfo[])this._stateInfos.toArray(new ParsedSynthStyle.StateInfo[size2]));
            this._stateInfos.clear();
        }
        this._style = null;
    }
    
    private void startState(Attributes attributes) throws SAXException {
        int n = 0;
        String value = null;
        this._stateInfo = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("id")) {
                value = attributes.getValue(i);
            }
            else if (qName.equals("idref")) {
                this._stateInfo = (ParsedSynthStyle.StateInfo)this.lookup(attributes.getValue(i), ParsedSynthStyle.StateInfo.class);
            }
            else if (qName.equals("clone")) {
                this._stateInfo = (ParsedSynthStyle.StateInfo)((ParsedSynthStyle.StateInfo)this.lookup(attributes.getValue(i), ParsedSynthStyle.StateInfo.class)).clone();
            }
            else if (qName.equals("value")) {
                StringTokenizer o = new StringTokenizer(attributes.getValue(i));
                while (o.hasMoreTokens()) {
                    String intern = o.nextToken().toUpperCase().intern();
                    if (intern == "ENABLED") {
                        n |= 0x1;
                    }
                    else if (intern == "MOUSE_OVER") {
                        n |= 0x2;
                    }
                    else if (intern == "PRESSED") {
                        n |= 0x4;
                    }
                    else if (intern == "DISABLED") {
                        n |= 0x8;
                    }
                    else if (intern == "FOCUSED") {
                        n |= 0x100;
                    }
                    else if (intern == "SELECTED") {
                        n |= 0x200;
                    }
                    else if (intern == "DEFAULT") {
                        n |= 0x400;
                    }
                    else {
                        if (intern != "AND") {
                            throw new SAXException("Unknown state: " + n);
                        }
                        continue;
                    }
                }
            }
        }
        if (this._stateInfo == null) {
            this._stateInfo = new ParsedSynthStyle.StateInfo();
        }
        this._stateInfo.setComponentState(n);
        this.register(value, this._stateInfo);
        this._stateInfos.add(this._stateInfo);
    }
    
    private void endState() {
        int size = this._statePainters.size();
        if (size > 0) {
            this._stateInfo.setPainters((ParsedSynthStyle.PainterInfo[])this._statePainters.toArray(new ParsedSynthStyle.PainterInfo[size]));
            this._statePainters.clear();
        }
        this._stateInfo = null;
    }
    
    private void startFont(Attributes attributes) throws SAXException {
        Object o = null;
        int style = 0;
        int int1 = 0;
        String value = null;
        String value2 = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("id")) {
                value = attributes.getValue(i);
            }
            else if (qName.equals("idref")) {
                o = (Object)this.lookup(attributes.getValue(i), Font.class);
            }
            else if (qName.equals("name")) {
                value2 = attributes.getValue(i);
            }
            else if (qName.equals("size")) {
                try {
                    int1 = Integer.parseInt(attributes.getValue(i));
                }
                catch (NumberFormatException ex) {
                    throw new SAXException("Invalid font size: " + attributes.getValue(i));
                }
            }
            else if (qName.equals("style")) {
                StringTokenizer o2 = new StringTokenizer(attributes.getValue(i));
                while (o2.hasMoreTokens()) {
                    String intern = o2.nextToken().intern();
                    if (intern == "BOLD") {
                        style = (((style | 0x0) ^ 0x0) | 0x1);
                    }
                    else if (intern == "ITALIC") {
                        style |= 0x2;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        if (o == null) {
            if (value2 == null) {
                throw new SAXException("You must define a name for the font");
            }
            if (int1 == 0) {
                throw new SAXException("You must define a size for the font");
            }
            o = new FontUIResource(value2, style, int1);
        }
        else if (value2 != null || int1 != 0 || style != 0) {
            throw new SAXException("Name, size and style are not for use with idref");
        }
        this.register(value, o);
        if (this._stateInfo != null) {
            this._stateInfo.setFont(o);
        }
        else if (this._style != null) {
            this._style.setFont(o);
        }
    }
    
    private void startColor(Attributes attributes) throws SAXException {
        Object o = null;
        String value = null;
        this._colorTypes.clear();
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("id")) {
                value = attributes.getValue(i);
            }
            else if (qName.equals("idref")) {
                o = (Object)this.lookup(attributes.getValue(i), Color.class);
            }
            else if (!qName.equals("name")) {
                if (qName.equals("value")) {
                    String value2 = attributes.getValue(i);
                    if (value2.startsWith("#")) {
                        try {
                            int length = value2.length();
                            int rgba;
                            int hasalpha;
                            if (length < 8) {
                                rgba = (int)Integer.decode(value2);
                                hasalpha = false;
                            }
                            else if (length == 8) {
                                rgba = (int)Integer.decode(value2);
                                hasalpha = true;
                            }
                            else if (length == 9) {
                                rgba = ((int)Integer.decode(value2.substring(0, 3)) << 24 | (int)Integer.decode(35 + value2.substring(3, 9)));
                                hasalpha = true;
                            }
                            else {
                                throw new SAXException("Invalid Color value: " + value2);
                            }
                            o = new ColorUIResource(new Color(rgba, (boolean)hasalpha));
                        }
                        catch (NumberFormatException ex6) {
                            throw new SAXException("Invalid Color value: " + value2);
                        }
                    }
                    else {
                        try {
                            o = new ColorUIResource((Color)Color.class.getField(value2.toUpperCase()).get(Color.class));
                        }
                        catch (NoSuchFieldException ex4) {
                            throw new SAXException("Invalid color name: " + value2);
                        }
                        catch (IllegalAccessException ex5) {
                            throw new SAXException("Invalid color name: " + value2);
                        }
                    }
                }
                else if (qName.equals("type")) {
                    StringTokenizer o2 = new StringTokenizer(attributes.getValue(i));
                    while (o2.hasMoreTokens()) {
                        String nextToken = o2.nextToken();
                        int lastIndex = nextToken.lastIndexOf(46);
                        Class<Object> forName;
                        if (lastIndex == -1) {
                            forName = ColorType.class;
                            lastIndex = 0;
                        }
                        else {
                            try {
                                forName = ReflectUtil.forName(nextToken.substring(0, lastIndex));
                            }
                            catch (ClassNotFoundException ex) {
                                throw new SAXException("Unknown class: " + nextToken.substring(0, lastIndex));
                            }
                            ++lastIndex;
                        }
                        try {
                            this._colorTypes.add((ColorType)this.checkCast(forName.getField(nextToken.substring(lastIndex)).get(forName), ColorType.class));
                        }
                        catch (NoSuchFieldException ex2) {
                            throw new SAXException("Unable to find color type: " + nextToken);
                        }
                        catch (IllegalAccessException ex3) {
                            throw new SAXException("Unable to find color type: " + nextToken);
                        }
                    }
                }
            }
        }
        if (o == null) {
            throw new SAXException("color: you must specificy a value");
        }
        this.register(value, o);
        if (this._stateInfo != null && this._colorTypes.size() > 0) {
            Color[] colors = this._stateInfo.getColors();
            int max = 0;
            for (int j = this._colorTypes.size() - 1; j >= 0; --j) {
                max = Math.max(max, ((ColorType)this._colorTypes.get(j)).getID());
            }
            if (colors == null || colors.length <= max) {
                Color[] array = new Color[max + 1];
                if (colors != null) {
                    System.arraycopy(colors, 0, array, 0, colors.length);
                }
                colors = array;
            }
            for (int k = this._colorTypes.size() - 1; k >= 0; --k) {
                colors[((ColorType)this._colorTypes.get(k)).getID()] = o;
            }
            this._stateInfo.setColors(colors);
        }
        return;
    }
    
    private void startProperty(Attributes attributes, Object o) throws SAXException {
        Object o2 = null;
        String value = null;
        int n = 0;
        String value2 = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("type")) {
                String upperCase = attributes.getValue(i).toUpperCase();
                if (upperCase.equals("IDREF")) {
                    n = 0;
                }
                else if (upperCase.equals("BOOLEAN")) {
                    n = 1;
                }
                else if (upperCase.equals("DIMENSION")) {
                    n = 2;
                }
                else if (upperCase.equals("INSETS")) {
                    n = 3;
                }
                else if (upperCase.equals("INTEGER")) {
                    n = 4;
                }
                else if (upperCase.equals("STRING")) {
                    n = 5;
                }
                else {
                    throw new SAXException(o + " unknown type, useidref, boolean, dimension, insets or integer");
                }
            }
            else if (qName.equals("value")) {
                value2 = attributes.getValue(i);
            }
            else if (qName.equals("key")) {
                value = attributes.getValue(i);
            }
        }
        if (value2 != null) {
            switch (n) {
                case 0: {
                    o2 = this.lookup(value2, Object.class);
                    break;
                }
                case 1: {
                    if (value2.toUpperCase().equals("TRUE")) {
                        o2 = Boolean.TRUE;
                        break;
                    }
                    o2 = Boolean.FALSE;
                    break;
                }
                case 2: {
                    StringTokenizer o3 = new StringTokenizer(value2);
                    o2 = new DimensionUIResource(this.nextInt(o3, "Invalid dimension"), this.nextInt(o3, "Invalid dimension"));
                    break;
                }
                case 3: {
                    o2 = this.parseInsets(value2, o + " invalid insets");
                    break;
                }
                case 4: {
                    try {
                        o2 = new Integer(Integer.parseInt(value2));
                        break;
                    }
                    catch (NumberFormatException ex) {
                        throw new SAXException(o + " invalid value");
                    }
                }
                case 5: {
                    o2 = value2;
                    break;
                }
            }
        }
        if (o2 == null || value == null) {
            throw new SAXException(o + ": you must supply a key and value");
        }
        if (o == "defaultsProperty") {
            this._defaultsMap.put(value, o2);
        }
        else if (this._stateInfo != null) {
            if (this._stateInfo.getData() == null) {
                this._stateInfo.setData(new HashMap());
            }
            this._stateInfo.getData().put(value, o2);
        }
        else if (this._style != null) {
            if (this._style.getData() == null) {
                this._style.setData(new HashMap());
            }
            this._style.getData().put(value, o2);
        }
        return;
    }
    
    private void startGraphics(Attributes attributes) throws SAXException {
        SynthGraphicsUtils graphicsUtils = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            if (attributes.getQName(i).equals("idref")) {
                graphicsUtils = (SynthGraphicsUtils)this.lookup(attributes.getValue(i), SynthGraphicsUtils.class);
            }
        }
        if (graphicsUtils == null) {
            throw new SAXException("graphicsUtils: you must supply an idref");
        }
        if (this._style != null) {
            this._style.setGraphicsUtils(graphicsUtils);
        }
        return;
    }
    
    private void startInsets(Attributes attributes) throws SAXException {
        int int1 = 0;
        int int2 = 0;
        int int3 = 0;
        int int4 = 0;
        Object insets = null;
        String value = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            try {
                if (qName.equals("idref")) {
                    insets = (Object)this.lookup(attributes.getValue(i), Insets.class);
                }
                else if (qName.equals("id")) {
                    value = attributes.getValue(i);
                }
                else if (qName.equals("top")) {
                    int1 = Integer.parseInt(attributes.getValue(i));
                }
                else if (qName.equals("left")) {
                    int3 = Integer.parseInt(attributes.getValue(i));
                }
                else if (qName.equals("bottom")) {
                    int2 = Integer.parseInt(attributes.getValue(i));
                }
                else if (qName.equals("right")) {
                    int4 = Integer.parseInt(attributes.getValue(i));
                }
            }
            catch (NumberFormatException ex) {
                throw new SAXException("insets: bad integer value for " + attributes.getValue(i));
            }
        }
        if (insets == null) {
            insets = new InsetsUIResource(int1, int3, int2, int4);
        }
        this.register(value, insets);
        if (this._style != null) {
            this._style.setInsets(insets);
        }
    }
    
    private void startBind(Attributes attributes) throws SAXException {
        ParsedSynthStyle parsedSynthStyle = null;
        String value = null;
        int n = -1;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("style")) {
                parsedSynthStyle = (ParsedSynthStyle)this.lookup(attributes.getValue(i), ParsedSynthStyle.class);
            }
            else if (qName.equals("type")) {
                String upperCase = attributes.getValue(i).toUpperCase();
                if (upperCase.equals("NAME")) {
                    n = 0;
                }
                else if (upperCase.equals("REGION")) {
                    n = 1;
                }
                else {
                    throw new SAXException("bind: unknown type " + upperCase);
                }
            }
            else if (qName.equals("key")) {
                value = attributes.getValue(i);
            }
        }
        if (parsedSynthStyle == null || value == null || n == -1) {
            throw new SAXException("bind: you must specify a style, type and key");
        }
        try {
            this._factory.addStyle(parsedSynthStyle, value, n);
        }
        catch (PatternSyntaxException ex) {
            throw new SAXException("bind: " + value + " is not a valid regular expression");
        }
        return;
    }
    
    private void startPainter(Attributes attributes, String s) throws SAXException {
        Insets insets = null;
        Insets insets2 = null;
        String s2 = null;
        boolean equals = true;
        boolean equals2 = true;
        Object o = null;
        String lowerCase = null;
        String s3 = null;
        int n = -1;
        boolean equals3 = false;
        boolean n2 = false;
        boolean n3 = false;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            String value = attributes.getValue(i);
            if (qName.equals("id")) {
                s3 = value;
            }
            else if (qName.equals("method")) {
                lowerCase = value.toLowerCase(Locale.ENGLISH);
            }
            else if (qName.equals("idref")) {
                o = (Object)this.lookup(value, SynthPainter.class);
            }
            else if (qName.equals("path")) {
                s2 = value;
            }
            else if (qName.equals("sourceInsets")) {
                insets = this.parseInsets(value, s + ": sourceInsets must be top left bottom right");
            }
            else if (qName.equals("destinationInsets")) {
                insets2 = this.parseInsets(value, s + ": destinationInsets must be top left bottom right");
            }
            else if (qName.equals("paintCenter")) {
                equals = value.toLowerCase().equals("true");
                n3 = true;
            }
            else if (qName.equals("stretch")) {
                equals2 = value.toLowerCase().equals("true");
                n2 = true;
            }
            else if (qName.equals("direction")) {
                String intern = value.toUpperCase().intern();
                if (intern == "EAST") {
                    n = 3;
                }
                else if (intern == "NORTH") {
                    n = 1;
                }
                else if (intern == "SOUTH") {
                    n = 5;
                }
                else if (intern == "WEST") {
                    n = 7;
                }
                else if (intern == "TOP") {
                    n = 1;
                }
                else if (intern == "LEFT") {
                    n = 2;
                }
                else if (intern == "BOTTOM") {
                    n = 3;
                }
                else if (intern == "RIGHT") {
                    n = 4;
                }
                else if (intern == "HORIZONTAL") {
                    n = 0;
                }
                else if (intern == "VERTICAL") {
                    n = 1;
                }
                else if (intern == "HORIZONTAL_SPLIT") {
                    n = 1;
                }
                else if (intern == "VERTICAL_SPLIT") {
                    n = 0;
                }
                else {
                    throw new SAXException(s + ": unknown direction");
                }
            }
            else if (qName.equals("center")) {
                equals3 = value.toLowerCase().equals("true");
            }
        }
        if (o == null) {
            if (s == "painter") {
                throw new SAXException(s + ": you must specify an idref");
            }
            if (insets == null && equals3 == 0) {
                throw new SAXException("property: you must specify sourceInsets");
            }
            if (s2 == null) {
                throw new SAXException("property: you must specify a path");
            }
            if (equals3 && (insets != null || insets2 != null || n3 || n2)) {
                throw new SAXException("The attributes: sourceInsets, destinationInsets, paintCenter and stretch  are not legal when center is true");
            }
            o = new ImagePainter((boolean)(equals2 == 0), (boolean)equals, insets, insets2, this.getResource(s2), (boolean)equals3);
        }
        this.register(s3, o);
        if (this._stateInfo != null) {
            this.addPainterOrMerge(this._statePainters, lowerCase, o, n);
        }
        else if (this._style != null) {
            this.addPainterOrMerge(this._stylePainters, lowerCase, o, n);
        }
    }
    
    private void addPainterOrMerge(List<ParsedSynthStyle.PainterInfo> list, String s, SynthPainter synthPainter, int n) {
        ParsedSynthStyle.PainterInfo painterInfo = new ParsedSynthStyle.PainterInfo(s, synthPainter, n);
        for (ParsedSynthStyle.PainterInfo painterInfo2 : list) {
            if (painterInfo.equalsPainter(painterInfo2)) {
                painterInfo2.addPainter(synthPainter);
                return;
            }
            continue;
        }
        list.add(painterInfo);
    }
    
    private void startImageIcon(Attributes attributes) throws SAXException {
        String value = null;
        String value2 = null;
        for (int i = attributes.getLength() - 1; i >= 0; --i) {
            String qName = attributes.getQName(i);
            if (qName.equals("id")) {
                value2 = attributes.getValue(i);
            }
            else if (qName.equals("path")) {
                value = attributes.getValue(i);
            }
        }
        if (value == null) {
            throw new SAXException("imageIcon: you must specify a path");
        }
        this.register(value2, new SynthParser.LazyImageIcon(this.getResource(value)));
        return;
    }
    
    private void startOpaque(Attributes attributes) {
        if (this._style != null) {
            this._style.setOpaque(true);
            for (int i = attributes.getLength() - 1; i >= 0; --i) {
                if (attributes.getQName(i).equals("value")) {
                    this._style.setOpaque("true".equals(attributes.getValue(i).toLowerCase()));
                }
            }
        }
    }
    
    private void startInputMap(Attributes attributes) throws SAXException {
        this._inputMapBindings.clear();
        this._inputMapID = null;
        if (this._style != null) {
            for (int i = attributes.getLength() - 1; i >= 0; --i) {
                if (attributes.getQName(i).equals("id")) {
                    this._inputMapID = attributes.getValue(i);
                }
            }
        }
    }
    
    private void endInputMap() throws SAXException {
        if (this._inputMapID != null) {
            this.register(this._inputMapID, new UIDefaults.LazyInputMap(this._inputMapBindings.toArray(new Object[this._inputMapBindings.size()])));
        }
        this._inputMapBindings.clear();
        this._inputMapID = null;
    }
    
    private void startBindKey(Attributes attributes) throws SAXException {
        if (this._inputMapID == null) {
            return;
        }
        if (this._style != null) {
            String value = null;
            String value2 = null;
            for (int i = attributes.getLength() - 1; i >= 0; --i) {
                String qName = attributes.getQName(i);
                if (qName.equals("key")) {
                    value = attributes.getValue(i);
                }
                else if (qName.equals("action")) {
                    value2 = attributes.getValue(i);
                }
            }
            if (value == null || value2 == null) {
                throw new SAXException("bindKey: you must supply a key and action");
            }
            this._inputMapBindings.add(value);
            this._inputMapBindings.add(value2);
        }
        return;
    }
    
    @Override
    public InputSource resolveEntity(String publicId, String systemId) throws IOException, SAXException {
        if (this.isForwarding()) {
            return this.getHandler().resolveEntity(publicId, systemId);
        }
        return null;
    }
    
    @Override
    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().notationDecl(name, publicId, systemId);
        }
    }
    
    @Override
    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().unparsedEntityDecl(name, publicId, systemId, notationName);
        }
    }
    
    @Override
    public void setDocumentLocator(Locator documentLocator) {
        if (this.isForwarding()) {
            this.getHandler().setDocumentLocator(documentLocator);
        }
    }
    
    @Override
    public void startDocument() throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().startDocument();
        }
    }
    
    @Override
    public void endDocument() throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().endDocument();
        }
    }
    
    @Override
    public void startElement(String uri, String localName, String intern, Attributes attributes) throws SAXException {
        intern = intern.intern();
        if (intern == "style") {
            this.startStyle(attributes);
        }
        else if (intern == "state") {
            this.startState(attributes);
        }
        else if (intern == "font") {
            this.startFont(attributes);
        }
        else if (intern == "color") {
            this.startColor(attributes);
        }
        else if (intern == "painter") {
            this.startPainter(attributes, intern);
        }
        else if (intern == "imagePainter") {
            this.startPainter(attributes, intern);
        }
        else if (intern == "property") {
            this.startProperty(attributes, "property");
        }
        else if (intern == "defaultsProperty") {
            this.startProperty(attributes, "defaultsProperty");
        }
        else if (intern == "graphicsUtils") {
            this.startGraphics(attributes);
        }
        else if (intern == "insets") {
            this.startInsets(attributes);
        }
        else if (intern == "bind") {
            this.startBind(attributes);
        }
        else if (intern == "bindKey") {
            this.startBindKey(attributes);
        }
        else if (intern == "imageIcon") {
            this.startImageIcon(attributes);
        }
        else if (intern == "opaque") {
            this.startOpaque(attributes);
        }
        else if (intern == "inputMap") {
            this.startInputMap(attributes);
        }
        else if (intern != "synth") {
            if (this._depth++ == 0) {
                this.getHandler().startDocument();
            }
            this.getHandler().startElement(uri, localName, intern, attributes);
        }
    }
    
    @Override
    public void endElement(String uri, String localName, String intern) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().endElement(uri, localName, intern);
            --this._depth;
            if (!this.isForwarding()) {
                this.getHandler().startDocument();
            }
        }
        else {
            intern = intern.intern();
            if (intern == "style") {
                this.endStyle();
            }
            else if (intern == "state") {
                this.endState();
            }
            else if (intern == "inputMap") {
                this.endInputMap();
            }
        }
    }
    
    @Override
    public void characters(char[] chars, int start, int length) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().characters(chars, start, length);
        }
    }
    
    @Override
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().ignorableWhitespace(ch, start, length);
        }
    }
    
    @Override
    public void processingInstruction(String target, String data) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().processingInstruction(target, data);
        }
    }
    
    @Override
    public void warning(SAXParseException e) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().warning(e);
        }
    }
    
    @Override
    public void error(SAXParseException e) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().error(e);
        }
    }
    
    @Override
    public void fatalError(SAXParseException e) throws SAXException {
        if (this.isForwarding()) {
            this.getHandler().fatalError(e);
        }
        throw e;
    }
    
    private static class LazyImageIcon extends ImageIcon implements UIResource
    {
        private URL location;
        
        public LazyImageIcon(URL location) {
            this.location = location;
        }
        
        @Override
        public void paintIcon(Component c, Graphics g, int x, int y) {
            if (this.getImage() != null) {
                super.paintIcon(c, g, x, y);
            }
        }
        
        @Override
        public int getIconWidth() {
            if (this.getImage() != null) {
                return super.getIconWidth();
            }
            return 0;
        }
        
        @Override
        public int getIconHeight() {
            if (this.getImage() != null) {
                return super.getIconHeight();
            }
            return 0;
        }
        
        @Override
        public Image getImage() {
            if (this.location != null) {
                this.setImage(Toolkit.getDefaultToolkit().getImage(this.location));
                this.location = null;
            }
            return super.getImage();
        }
    }
}
